/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor;

import static java.util.stream.Collectors.toList;
import static org.eclipse.core.runtime.IStatus.ERROR;
import static org.eclipse.emf.ecore.util.EcoreUtil.delete;
import static org.eclipse.jface.dialogs.ErrorDialog.openError;
import static org.fortiss.tooling.base.ui.editor.ConstraintBasedProcessEditorHelper.adjustColorIndex;
import static org.fortiss.tooling.base.ui.utils.ConstraintsBaseUIUtils.activateConfiguration;
import static org.fortiss.tooling.base.ui.utils.ConstraintsBaseUIUtils.deactivateConfiguration;
import static org.fortiss.tooling.base.utils.ConstraintsBaseUtils.DEFAULT_CONFIGURATION_NAME;
import static org.fortiss.tooling.base.utils.ConstraintsBaseUtils.getActiveConfigurationsTransitively;
import static org.fortiss.tooling.base.utils.ConstraintsBaseUtils.getActiveConstraintsTransitively;
import static org.fortiss.tooling.base.utils.ConstraintsBaseUtils.retrieveDefaultConfig;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.disableNotificationsOf;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.enableNotificationsOf;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ICheckStateProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TreeColumn;
import org.fortiss.tooling.base.model.element.ConstraintConfiguration;
import org.fortiss.tooling.base.model.element.ElementFactory;
import org.fortiss.tooling.base.model.element.IConstraintBasedProcess;
import org.fortiss.tooling.base.ui.ToolingBaseUIActivator;
import org.fortiss.tooling.base.ui.editor.ConstraintBasedProcessEditorHelper.ConfigurationRefWithContextualConfiguration;
import org.fortiss.tooling.base.ui.editor.ConstraintBasedProcessEditorHelper.ConstraintWithContextualConfiguration;
import org.fortiss.tooling.base.ui.editor.ConstraintBasedProcessEditorHelper.GreenFontStyler;
import org.fortiss.tooling.base.ui.editor.ConstraintBasedProcessEditorHelper.NormalFontStyler;
import org.fortiss.tooling.base.ui.editor.ConstraintBasedProcessEditorHelper.RedFontStyler;
import org.fortiss.tooling.base.ui.editor.ConstraintBasedProcessEditorHelper.TreeStructureNode;
import org.fortiss.tooling.kernel.extension.IConstraint;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.FailedConstraintInstanceStatus;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceContainer;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceStatus;
import org.fortiss.tooling.kernel.model.constraints.SuccessConstraintInstanceStatus;
import org.fortiss.tooling.kernel.model.constraints.impl.ConstraintInstanceImpl;
import org.fortiss.tooling.kernel.model.constraints.impl.ConstraintNameToChecksumImpl;
import org.fortiss.tooling.kernel.service.IConstraintService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.ui.extension.base.EditorBase;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;
import org.fortiss.tooling.kernel.ui.service.IActionService;
import org.fortiss.tooling.kernel.ui.service.IConstraintUIService;
import org.fortiss.tooling.kernel.ui.service.IContextMenuService;

// TODO Refactor this huge class and provide a dedicated perspective (see #3189)
/**
 * Editor for {@link IConstraintBasedProcess}s.
 * 
 * @author aravantinos
 */
public class ConstraintBasedProcessEditor<CBP extends IConstraintBasedProcess>
		extends EditorBase<CBP> implements ContextMenuContextProvider, ISelectionProvider {

	/** Width of columns to display constraints. */
	private static final int COLUMN_WIDTH = 1300;

	/** {@inheritDoc} */
	@Override
	public void addPartPropertyListener(IPropertyChangeListener listener) {
		super.addPartPropertyListener(listener);
	}

	/** Model context of the edited project. */
	private ITopLevelElement top;

	/** Constraint container for the edited project. */
	private IConstraintInstanceContainer cstrContainer;

	/** Tree viewer for the configurations. */
	private CheckboxTreeViewer treeViewer;

	/** Selection of the tree. */
	private ISelection selection;

	/** List of parent nodes. */
	private List<TreeStructureNode> parentList;

	/** Adapter for refreshing content. */
	private EContentAdapter refreshAdapter;

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		top = IPersistencyService.getInstance().getTopLevelElementFor(editedObject);
		getSite().setSelectionProvider(this);
		retrieveDefaultConfig(editedObject);
		cstrContainer = editedObject.getConstraintInstanceContainer();
		parent.setLayout(new GridLayout(1, true));
		createTree(parent);
		MenuManager menuManager = IContextMenuService.getInstance().createDefaultContextMenu(this);
		Menu menu = menuManager.createContextMenu(treeViewer.getTree());
		treeViewer.getTree().setMenu(menu);
		createCurrentObjectivePart(parent);
		refreshAdapter = new EContentAdapter() {
			/** {@inheritDoc} */
			@Override
			public void notifyChanged(Notification notification) {
				treeViewer.refresh();
			}
		};
		editedObject.eAdapters().add(refreshAdapter);
		treeViewer.addFilter(new ViewerFilter() {
			/** {@inheritDoc} */
			@Override
			public boolean select(Viewer viewer, Object parentElement, Object element) {
				if(element instanceof TreeStructureNode &&
						parentElement instanceof TreeStructureNode) {
					return checkIfDuplicate(viewer, parentElement, element);
				}
				return true;
			}
		});
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		disableNotificationsOf(editedObject);
		editedObject.eAdapters().remove(refreshAdapter);
		enableNotificationsOf(editedObject);
		super.dispose();
	}

	/** Create current the controls "Current objective:". */
	private void createCurrentObjectivePart(Composite parent) {
		Composite currentObjComposite = new Composite(parent, SWT.None);
		currentObjComposite.setLayout(new GridLayout(2, false));
	}

	/** Dummy object used to add new elements in the tree. */
	private static class DummyObject {
		// Nothing to implement: just a marker class
	}

	/** Content provider for the configuration tree. */
	private class ConfigTreeContentProvider extends TreeContentProviderBase {

		/** {@inheritDoc} */
		@Override
		public Object getParent(Object inputElement) {
			if(inputElement instanceof ConfigurationRefWithContextualConfiguration) {
				ConfigurationRefWithContextualConfiguration input =
						(ConfigurationRefWithContextualConfiguration)inputElement;
				for(TreeStructureNode iterableElement : parentList) {
					if(iterableElement.data instanceof ConfigurationRefWithContextualConfiguration) {
						ConfigurationRefWithContextualConfiguration ref =
								(ConfigurationRefWithContextualConfiguration)iterableElement.data;
						if(ref.config.getName().equalsIgnoreCase(input.config.getName())) {
							return iterableElement.parent;
						}
					}
				}
			} else if(inputElement instanceof TreeStructureNode) {
				return ((TreeStructureNode)inputElement).parent;
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public Object[] getChildren(Object parentElement) {

			if(parentList == null) {
				parentList = new ArrayList<TreeStructureNode>();
			}
			if(parentElement instanceof IConstraintBasedProcess) {
				IConstraintBasedProcess cbdp = (IConstraintBasedProcess)parentElement;
				ArrayList<Object> configs = new ArrayList<>();
				configs.add(new DummyObject());
				configs.addAll(cbdp.getConfigurations());
				return configs.toArray();
			} else if(parentElement instanceof ConstraintConfiguration) {

				List<Object> types = new ArrayList<Object>();
				final IConstraintUIService cuis = IConstraintUIService.getInstance();
				Set<Class<? extends IConstraint>> availableCstrs = cuis.getAllConstraints();
				availableCstrs.removeAll(cuis.getAlwaysActivatedConstraints());

				EList<ConstraintConfiguration> configs = editedObject.getConfigurations();
				if(configs.size() > 1) {
					List<ConfigurationRefWithContextualConfiguration> configCollection =
							configs.stream()
									.map(t -> new ConfigurationRefWithContextualConfiguration(t,
											(ConstraintConfiguration)parentElement))
									.collect(toList());
					List<ConfigurationRefWithContextualConfiguration> refs = configCollection;
					refs.removeIf(c -> c.config.getName().equalsIgnoreCase(c.target.getName()));
					refs.forEach(c -> {
						parentList.add(new TreeStructureNode(parentElement, c));
					});
					types.add(refs);
				}

				Set<Class<? extends IConstraint>> availableConstraints = cuis.getAllConstraints();

				// Collecting the groups in the array list
				List<String> groupName = new ArrayList<String>();
				for(Class<? extends IConstraint> c : availableConstraints) {
					String constGroupName = IConstraintService.getInstance().getGroupName(c);
					if(!groupName.contains(constGroupName)) {
						groupName.add(constGroupName);
					}
				}

				// Collect the constraints for each of the groups.
				for(String constraintGroupName : groupName) {
					HashMap<String, String> constraintInfo = new HashMap<String, String>();
					for(Class<? extends IConstraint> c : availableConstraints) {
						String cGroupName = IConstraintService.getInstance().getGroupName(c);
						String desc = cuis.getDescription(c);
						constraintMap.put(c.getName(), desc);
						if(cGroupName.equalsIgnoreCase(constraintGroupName)) {
							constraintInfo.put(c.getName(), cuis.getDescription(c));
						}
					}
					groupNameAndConstraints.put(constraintGroupName, constraintInfo);
				}
				// Add the constraint groups to the tree.
				for(String constraintGroupName : groupNameAndConstraints.keySet()) {
					types.add(new TreeStructureNode(parentElement, constraintGroupName));
					parentList.add(new TreeStructureNode(parentElement, constraintGroupName));
				}

				return types.toArray();
			} else if(parentElement instanceof List<?>) {
				return ((List<?>)parentElement).toArray();
			}

			// Add the constraints to the tree based on the group name.
			else if(parentElement instanceof TreeStructureNode) {
				HashMap<String, String> constraintInfo = new HashMap<String, String>();
				List<TreeStructureNode> nodeList = new ArrayList<TreeStructureNode>();

				Object stringNode = ((TreeStructureNode)parentElement).data;
				if(groupNameAndConstraints.get(stringNode) != null) {
					for(String constraintGroupName : groupNameAndConstraints.keySet()) {
						if(stringNode.toString().equalsIgnoreCase(constraintGroupName.toString())) {

							HashMap<String, String> groupedConstraint =
									groupNameAndConstraints.get(constraintGroupName);
							for(String key : groupedConstraint.keySet()) {
								constraintInfo.put(key, groupedConstraint.get(key));
								TreeStructureNode node = new TreeStructureNode(parentElement, key);
								parentList.add(node);
								nodeList.add(node);
							}
						}
					}
				}
				return nodeList.toArray();
			}
			return new Object[0];
		}
	}

	/** This constraint map contains the name and the description of all of the constraints */
	private HashMap<String, String> constraintMap = new HashMap<String, String>();

	/** This HashMap contains the groups and the the grouped constraints (as a HashMap) */
	private HashMap<String, HashMap<String, String>> groupNameAndConstraints =
			new HashMap<String, HashMap<String, String>>();

	/** Text labels for the tree of configurations. */
	private String getLabelText(Object element) {
		if(element instanceof DummyObject) {
			return "click to add a new configuration";
		} else if(element instanceof ConstraintConfiguration) {
			return ((ConstraintConfiguration)element).getName();
		} else if(element instanceof ConstraintWithContextualConfiguration) {
			IConstraintUIService cuis = IConstraintUIService.getInstance();
			return cuis.getDescription(((ConstraintWithContextualConfiguration)element).cstr);
		} else if(element instanceof ConfigurationRefWithContextualConfiguration) {
			return ((ConfigurationRefWithContextualConfiguration)element).target.getName();
		} else if(element instanceof List<?>) {
			return "Required configurations";
		} else if(element instanceof TreeStructureNode) {

			Object stringNode = ((TreeStructureNode)element).data;
			if(groupNameAndConstraints.containsKey(stringNode)) {
				for(String constraintGroupName : groupNameAndConstraints.keySet()) {
					if((stringNode.toString().equalsIgnoreCase(constraintGroupName.toString()))) {
						return constraintGroupName + getInstanceCount(element);
					}
				}
			} else if(constraintMap.containsKey(stringNode)) {
				return constraintMap.get(stringNode);
			}
		}
		return null;
	}

	/** Check state provider for the tree of configurations. */
	private class ConfigTreeCheckStateProvider implements ICheckStateProvider {

		/** {@inheritDoc} */
		@Override
		public boolean isGrayed(Object element) {
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public boolean isChecked(Object element) {

			if(element instanceof ConstraintWithContextualConfiguration) {
				ConstraintWithContextualConfiguration cstrInContext =
						(ConstraintWithContextualConfiguration)element;
				IConstraintService cs = IConstraintService.getInstance();
				String name = cs.getName(cstrInContext.cstr);
				HashMap<String, String> hashMap = groupNameAndConstraints.get(element.toString());
				return hashMap.keySet().contains(name);
			} else if(element instanceof ConfigurationRefWithContextualConfiguration) {

				ConfigurationRefWithContextualConfiguration configRef =
						(ConfigurationRefWithContextualConfiguration)element;
				return configRef.config.getIncludedConfigurations().contains(configRef.target);
			} else if(element instanceof TreeStructureNode) {
				String stringNode = ((TreeStructureNode)element).data.toString();
				if(constraintMap.containsKey(stringNode)) {
					ITreeContentProvider provider =
							(ITreeContentProvider)treeViewer.getContentProvider();
					Object parent = provider.getParent(element);
					Object parent2 = provider.getParent(parent);
					ConstraintConfiguration config = ((ConstraintConfiguration)parent2);
					EList<String> activeConstraints = config.getActiveConstraints();
					for(String string : activeConstraints) {
						if(string.equalsIgnoreCase(stringNode)) {
							return true;
						}
					}
				}
			}
			return false;
		}
	}

	/** Check state listener for the tree of configurations. */
	private class ConfigTreeCheckStateListener implements ICheckStateListener {
		/** {@inheritDoc} */
		@Override
		public void checkStateChanged(CheckStateChangedEvent event) {
			// Get object element from the state changed
			Object element = event.getElement();
			IConstraintService cs = IConstraintService.getInstance();
			IConstraintUIService cuis = IConstraintUIService.getInstance();
			ConstraintConfiguration currentObj = editedObject.getCurrentObjective();
			if(element instanceof ConstraintWithContextualConfiguration) {
				ConstraintWithContextualConfiguration cstrInContext =
						(ConstraintWithContextualConfiguration)element;
				String name = cs.getName(cstrInContext.cstr);

				top.runAsCommand(() -> {
					if(event.getChecked()) {

						cstrInContext.config.getActiveConstraints().add(name);
						List<ConstraintConfiguration> activeConfigs =
								getActiveConfigurationsTransitively(currentObj, null);
						if(activeConfigs.contains(cstrInContext.config)) {
							cuis.activate(cstrInContext.cstr, cstrContainer);
						}
					} else {
						cstrInContext.config.getActiveConstraints().remove(name);
						List<Class<? extends IConstraint>> stillActiveCstrs =
								getActiveConstraintsTransitively(currentObj, cstrInContext.config);
						if(!stillActiveCstrs.contains(cstrInContext.cstr)) {
							cuis.deactivate(cstrInContext.cstr, cstrContainer);
						}
					}
				});
			} else if(element instanceof ConfigurationRefWithContextualConfiguration) {

				ConfigurationRefWithContextualConfiguration configRef =
						(ConfigurationRefWithContextualConfiguration)element;
				top.runAsCommand(() -> {
					if(event.getChecked()) {
						configRef.config.getIncludedConfigurations().add(configRef.target);
						activateConfiguration(configRef.target, cstrContainer);
					} else {
						configRef.config.getIncludedConfigurations().remove(configRef.target);
						deactivateConfiguration(configRef.target, cstrContainer, configRef.config);
					}
				});
			}
			// Check if the constraint belongs to the constraints HashMap
			else if(element instanceof TreeStructureNode) {

				ITreeContentProvider provider =
						(ITreeContentProvider)treeViewer.getContentProvider();
				Object parent = provider.getParent(element);
				Object parent2 = provider.getParent(parent);
				ConstraintConfiguration config = ((ConstraintConfiguration)parent2);
				String constraintName = ((TreeStructureNode)element).data.toString();
				if(constraintMap.containsKey(constraintName)) {
					top.runAsNonDirtyingCommand(new Runnable() {
						/** {@inheritDoc} */
						@Override
						public void run() {
							// Get the constraintName as a full class path (e.g.,
							// a.b.c.constraintName.Java)
							if(event.getChecked()) {
								config.getActiveConstraints().add(constraintName);

								List<ConstraintConfiguration> activeConfigs =
										getActiveConfigurationsTransitively(config, null);
								// If the set of active constraints is not empty, and the selected
								// constraint is in the active constraints list
								if(!activeConfigs.isEmpty() && activeConfigs.get(0) != null) {
									// get all active constraints of the first configuration which
									// is selected using activeConfigs.get(0)
									EList<String> activeConstraints =
											activeConfigs.get(0).getActiveConstraints();
									if(activeConstraints
											.contains(cs.getConstraintByName(constraintName)
													.getCanonicalName())) {
										// Activate the constraint
										cuis.activate(cs.getConstraintByName(constraintName),
												cstrContainer);
									}
								}
							} else {
								clearConstraintInstance(cs, cuis, config, constraintName);
							}
						}
					});
				}
			}
			treeViewer.refresh();
		}
	}

	/** Tree for configurations. */
	private class ConfigTree extends CheckboxTreeViewer implements ISelectionChangedListener {

		/** Constructor. */
		public ConfigTree(Composite parent) {
			super(parent);
			getTree().setLayoutData(new GridData(800, 600));
			setContentProvider(new ConfigTreeContentProvider());
			setCheckStateProvider(new ConfigTreeCheckStateProvider());
			addCheckStateListener(new ConfigTreeCheckStateListener());
		}

		/** {@inheritDoc} */
		@Override
		public void selectionChanged(SelectionChangedEvent event) {
			selection = event.getSelection();
			IActionService.getInstance().refresh();
		}
	}

	/** Creates the tree of configurations. */
	private void createTree(Composite parent) {
		treeViewer = new ConfigTree(parent);
		TreeColumn column = new TreeColumn(treeViewer.getTree(), SWT.LEFT);
		column.setText("Constraint configurations");
		column.setWidth(COLUMN_WIDTH);
		TreeViewerColumn onlyViewerColumn = new TreeViewerColumn(treeViewer, column);
		onlyViewerColumn.setLabelProvider(new BoldColumnLabelProvider());

		onlyViewerColumn.setEditingSupport(new EditingSupport(treeViewer) {
			/** {@inheritDoc} */
			@Override
			protected void setValue(Object element, Object value) {
				if(element instanceof DummyObject) {
					if(!alreadyExists((String)value)) {
						ConstraintConfiguration newconfig =
								ElementFactory.eINSTANCE.createConstraintConfiguration();
						newconfig.setName((String)value);
						top.runAsCommand(() -> IElementCompositorService.getInstance()
								.compose(editedObject, newconfig, null));
					}
				}
				if(element instanceof ConstraintConfiguration) {
					ConstraintConfiguration config = (ConstraintConfiguration)element;
					if(config.getName().equals(DEFAULT_CONFIGURATION_NAME) ||
							config.getName().equals(value)) {
						return;
					}
					if(!alreadyExists((String)value)) {
						top.runAsCommand(
								() -> ((ConstraintConfiguration)element).setName((String)value));
					}
				}
			}

			/** {@inheritDoc} */
			@Override
			protected Object getValue(Object element) {
				return getLabelText(element);
			}

			/** {@inheritDoc} */
			@Override
			protected CellEditor getCellEditor(Object element) {
				return new TextCellEditor((Composite)getViewer().getControl());
			}

			/** {@inheritDoc} */
			@Override
			protected boolean canEdit(Object element) {
				return element instanceof ConstraintConfiguration ||
						element instanceof DummyObject || constraintMap.containsKey(element);
			}
		});
		treeViewer.setUseHashlookup(true);
		treeViewer.setComparator(new ConstraintComparator());
		treeViewer.setInput(editedObject);
	}

	/**
	 * @param element
	 *            the element to set color
	 * @return color instance that will be applied on the element
	 */
	public Color getConstraintBackgroundColor(Object element) {
		// Get the complete path of the constraint class (e.g., a.b.c.constraint.java)
		String constraintToFind = ((TreeStructureNode)element).data.toString();
		// If the constraint is in the active constraint list and the status is true
		// than make the background color green

		EList<ConstraintInstance> constraintInstances =
				editedObject.getConstraintInstanceContainer().getConstraintInstances();
		ITreeContentProvider provider = (ITreeContentProvider)treeViewer.getContentProvider();
		Object parent = provider.getParent(element);
		Object parent2 = provider.getParent(parent);
		ConstraintConfiguration config = ((ConstraintConfiguration)parent2);

		final Display display = Display.getCurrent();
		if(config.getName().equalsIgnoreCase(editedObject.getCurrentObjective().getName())) {
			for(ConstraintInstance instance : constraintInstances) {
				for(String constraintName : editedObject.getCurrentObjective()
						.getActiveConstraints()) {
					if(instance.getConstraintName().equalsIgnoreCase(constraintName) &&
							constraintToFind.equalsIgnoreCase(constraintName)) {
						IConstraintInstanceStatus status =
								IConstraintUIService.getInstance().getStatus(instance);
						if(status instanceof SuccessConstraintInstanceStatus) {
							return display.getSystemColor(SWT.COLOR_GREEN);
						} else if(status instanceof FailedConstraintInstanceStatus) {
							return display.getSystemColor(SWT.COLOR_RED);
						}
					}
				}
			}
		}
		return display.getSystemColor(SWT.COLOR_WHITE);
	}

	/**
	 * Checks if the name already exists among current configurations, if so displays an error
	 * message and return true, otherwise returns false.
	 */
	private boolean alreadyExists(String name) {
		List<String> configNames =
				editedObject.getConfigurations().stream().map(c -> c.getName()).collect(toList());
		if(configNames.contains(name)) {
			Status status = new Status(ERROR, ToolingBaseUIActivator.PLUGIN_ID, 0,
					"A configuration with this name already exists.", null);
			openError(Display.getCurrent().getActiveShell(), "Error", null, status);
			return true;
		}
		return false;
	}

	/** Comparator for the contents of the tree. */
	private class ConstraintComparator extends ViewerComparator {

		/** {@inheritDoc} */
		@Override
		public int compare(Viewer viewer, Object e1, Object e2) {
			if(e1 instanceof DummyObject) {
				return -1;
			} else if(e2 instanceof DummyObject) {
				return 1;
			} else if(e1 instanceof ConstraintConfiguration &&
					e2 instanceof ConstraintConfiguration) {
				return ((ConstraintConfiguration)e1).getName()
						.compareToIgnoreCase(((ConstraintConfiguration)e2).getName());
			} else if(e1 instanceof ConstraintWithContextualConfiguration &&
					e2 instanceof ConstraintWithContextualConfiguration) {
				IConstraintUIService cuis = IConstraintUIService.getInstance();
				String desc1 =
						cuis.getDescription(((ConstraintWithContextualConfiguration)e1).cstr);
				String desc2 =
						cuis.getDescription(((ConstraintWithContextualConfiguration)e2).cstr);
				return desc1.compareToIgnoreCase(desc2);
			} else if(e1 instanceof ConfigurationRefWithContextualConfiguration &&
					e2 instanceof ConfigurationRefWithContextualConfiguration) {
				String n1 = ((ConfigurationRefWithContextualConfiguration)e1).target.getName();
				String n2 = ((ConfigurationRefWithContextualConfiguration)e2).target.getName();
				return n1.compareToIgnoreCase(n2);
			} else if(e1 instanceof TreeStructureNode && e2 instanceof TreeStructureNode) {
				String n1 = ((TreeStructureNode)e1).parent.toString();
				String n2 = ((TreeStructureNode)e2).parent.toString();
				return n1.compareToIgnoreCase(n2);
			}
			return super.compare(viewer, e1, e2);
		}
	}

	/** {@inheritDoc} */
	@Override
	public EObject getSelectedModelElement() {
		if(getSelectedModelElementList().isEmpty()) {
			return null;
		}
		// Returns the first element, which is ensured to exist by the previous statement.
		Object selected = getSelectedModelElementList().get(0);
		return selected instanceof EObject ? (EObject)selected : null;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public List<EObject> getSelectedModelElementList() {
		boolean isStructured = selection instanceof StructuredSelection;
		return isStructured ? ((StructuredSelection)selection).toList() : new ArrayList<>();
	}

	/** {@inheritDoc} */
	@Override
	public void addSelectionChangedListener(ISelectionChangedListener listener) {
		// Not supported
	}

	/** {@inheritDoc} */
	@Override
	public ISelection getSelection() {
		return selection == null ? new StructuredSelection() : selection;
	}

	/** {@inheritDoc} */
	@Override
	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
		// Not supported
	}

	/** {@inheritDoc} */
	@Override
	public void setSelection(ISelection selection) {
		// Not supported
	}

	/**
	 * If the constraint is unchecked from the tree remove the
	 * constraint from the active constraints list and constraint instance list
	 * 
	 * @param cs
	 *            constraintService
	 * @param cuis
	 *            constraintUIService
	 * @param config
	 *            the configuration in which the constraint remains
	 * @param constraintName
	 *            the name of the constraint
	 */
	private void clearConstraintInstance(IConstraintService cs, IConstraintUIService cuis,
			ConstraintConfiguration config, String constraintName) {

		config.getActiveConstraints().remove(constraintName);
		// If the constraint is still not active
		EList<String> stillActiveConstraints = config.getActiveConstraints();
		if(!stillActiveConstraints
				.contains(cs.getConstraintByName(constraintName).getCanonicalName())) {
			Class<? extends IConstraint> constrainttoDeActivate =
					cs.getConstraintByName(constraintName);
			// Deactivate the constraint
			cuis.deactivate(constrainttoDeActivate, cstrContainer);

			ITopLevelElement modelContext =
					IPersistencyService.getInstance().getTopLevelElementFor(editedObject);
			modelContext.runAsNonDirtyingCommand(() -> {

				TreeIterator<EObject> allContents =
						editedObject.getCurrentObjective().eResource().getAllContents();
				EList<EObject> eContents = editedObject.getCurrentObjective().eContents();
				List<EObject> duplicateList = new ArrayList<EObject>();
				duplicateList.addAll(eContents);
				List<Object> toRemove = new ArrayList<Object>();

				while(allContents.hasNext()) {
					EObject next = allContents.next();

					if(next instanceof ConstraintNameToChecksumImpl &&
							next.toString().contains(constraintName)) {
						toRemove.add(next);
					}
					if(next instanceof ConstraintInstanceImpl &&
							next.toString().contains(constraintName)) {
						toRemove.add(next);
					}
				}
				toRemove.forEach(c -> {
					duplicateList.remove(c);
					delete((EObject)c);
				});
			});
		}
	}

	/**
	 * @param viewer
	 *            the tree viewer
	 * @param parentElement
	 *            the parent element of the given element
	 * @param element
	 *            the element to consider
	 * @return the decision on whether the constraint was checked in previous dependent configs or
	 *         not
	 */
	private boolean checkIfDuplicate(Viewer viewer, Object parentElement, Object element) {
		ITreeContentProvider provider = (ITreeContentProvider)treeViewer.getContentProvider();

		ConstraintConfiguration config =
				((ConstraintConfiguration)provider.getParent(parentElement));
		List<Class<? extends IConstraint>> activeConstraintsTransitively =
				getActiveConstraintsTransitively(config, null);
		List<String> constraintList = activeConstraintsTransitively.stream()
				.map((item) -> item.getName()).collect(toList());

		for(String string : constraintList) {
			if(!config.getActiveConstraints().contains(string) &&
					((TreeStructureNode)element).data.toString().equalsIgnoreCase(string)) {
				return false;
			}
		}

		return true;
	}

	/** @return the concatenated string containing the score for the configuration. */
	private String getInstanceCount(Object element) {
		String constraintGroupName = ((TreeStructureNode)element).data.toString();
		// If the constraint is in the active constraint list and the status is true
		// than make the background color green

		int successCount = 0;
		int failCount = 0;
		HashMap<String, String> hashMap = groupNameAndConstraints.get(constraintGroupName);
		EList<ConstraintInstance> constraintInstances =
				editedObject.getConstraintInstanceContainer().getConstraintInstances();
		ITreeContentProvider provider = (ITreeContentProvider)treeViewer.getContentProvider();
		Object parent = provider.getParent(element);
		ConstraintConfiguration config = ((ConstraintConfiguration)parent);

		EList<String> activeConstraints = config.getActiveConstraints();
		List<Class<? extends IConstraint>> stillActiveCstrs =
				getActiveConstraintsTransitively(config, null);

		int duplicateSize = stillActiveCstrs.stream()
				.filter(c -> !(activeConstraints.contains(c.getName()))).collect(toList()).size();
		if(config.getName().equalsIgnoreCase(editedObject.getCurrentObjective().getName())) {
			for(ConstraintInstance instance : constraintInstances) {
				for(String constraintName : editedObject.getCurrentObjective()
						.getActiveConstraints()) {
					if(instance.getConstraintName().equalsIgnoreCase(constraintName) &&
							(hashMap.containsKey(constraintName))) {
						IConstraintInstanceStatus status =
								IConstraintUIService.getInstance().getStatus(instance);
						if(status instanceof SuccessConstraintInstanceStatus) {
							successCount++;
						}
					}
				}
			}

			int totalNumOfGrpConstraints = hashMap.size() - duplicateSize;
			return "[ " + successCount + " / " + failCount + "/" + totalNumOfGrpConstraints + " ]";
		}
		return "";
	}

	/** Text labels for the tree of configurations. */
	private StyledString getStylizedLabel(Object element) {
		if(element instanceof DummyObject) {
			return new StyledString("click to add a new configuration");
		} else if(element instanceof ConstraintConfiguration) {
			return new StyledString(((ConstraintConfiguration)element).getName());
		} else if(element instanceof ConstraintBasedProcessEditorHelper.ConstraintWithContextualConfiguration) {
			IConstraintUIService cuis = IConstraintUIService.getInstance();
			return new StyledString(cuis.getDescription(
					((ConstraintBasedProcessEditorHelper.ConstraintWithContextualConfiguration)element).cstr));
		} else if(element instanceof ConfigurationRefWithContextualConfiguration) {
			return new StyledString(
					((ConfigurationRefWithContextualConfiguration)element).target.getName());
		} else if(element instanceof List<?>) {
			return new StyledString("Required configurations");
		} else if(element instanceof TreeStructureNode) {

			Object stringNode = ((TreeStructureNode)element).data;
			if(groupNameAndConstraints.containsKey(stringNode)) {
				for(String constraintGroupName : groupNameAndConstraints.keySet()) {
					if((stringNode.toString().equalsIgnoreCase(constraintGroupName.toString()))) {

						ArrayList<Integer> instanceCountList = getInstanceCount2(element);
						int constarintNameLength = constraintGroupName.length();
						if(instanceCountList == null) {
							StyledString finalString = new StyledString(constraintGroupName);
							adjustColorIndex(groupNameAndConstraints.keySet().size());
							finalString.setStyle(0, finalString.length(), new NormalFontStyler());
							return finalString;
						}
						// the elements are stored in the list sequentially- success count, fail
						// count and total number of instances
						String successfulConst = Integer.toString(instanceCountList.get(0));
						int successLength = successfulConst.length();
						String failedConst = Integer.toString(instanceCountList.get(1));
						String totalConst = Integer.toString(instanceCountList.get(2));
						int failCountLength = failedConst.length();
						// creating the string with all the counts
						if(instanceCountList.size() == 3) {

							StyledString finalString = new StyledString(constraintGroupName + "[" +
									successfulConst + "/" + failedConst + "/" + totalConst + "]");
							adjustColorIndex(groupNameAndConstraints.keySet().size());

							finalString.setStyle(0, finalString.length(), new NormalFontStyler());
							finalString.setStyle(constarintNameLength + 1, successLength,
									new GreenFontStyler());
							finalString.setStyle(constarintNameLength + 2 + successLength,
									failCountLength, new RedFontStyler());
							return finalString;
						}
					}
				}
			} else if(constraintMap.containsKey(stringNode)) {
				return new StyledString(constraintMap.get(stringNode));
			}
		}
		return null;
	}

	/** Class to create stylized string for labels of the tree. */
	private class BoldColumnLabelProvider extends DelegatingStyledCellLabelProvider {
		/** {@inheritDoc} */
		@Override
		public Color getForeground(Object element) {
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public Color getBackground(Object element) {
			if(element instanceof TreeStructureNode) {
				if(constraintMap.containsKey(((TreeStructureNode)element).data.toString())) {
					// Ask for the constraint color based on status
					return getConstraintBackgroundColor(element);
				}
			}
			return null;
		}

		/** Class for generating stylized text. */
		public BoldColumnLabelProvider() {
			super(new IStyledLabelProvider() {
				/** {@inheritDoc} */
				@Override
				public Image getImage(Object element) {
					return null;
				}

				/** {@inheritDoc} */
				@Override
				public StyledString getStyledText(Object element) {
					return getStylizedLabel(element);
				}

				/** {@inheritDoc} */
				@Override
				public void addListener(ILabelProviderListener listener) {
					return;
				}

				/** {@inheritDoc} */
				@Override
				public void dispose() {
					return;
				}

				/** {@inheritDoc} */
				@Override
				public boolean isLabelProperty(Object element, String property) {
					return false;
				}

				/** {@inheritDoc} */
				@Override
				public void removeListener(ILabelProviderListener listener) {
					return;
				}
			});
		}
	}

	/**
	 * @param element
	 *            the group name element
	 * @return array list of integer containing success count, fail count, and total count
	 */
	private ArrayList<Integer> getInstanceCount2(Object element) {
		String constraintGroupName = ((TreeStructureNode)element).data.toString();
		// If the constraint is in the active constraint list and the status is true
		// than make the background color green

		int successCount = 0;
		int failCount = 0;
		HashMap<String, String> hashMap = groupNameAndConstraints.get(constraintGroupName);
		EList<ConstraintInstance> constraintInstances =
				editedObject.getConstraintInstanceContainer().getConstraintInstances();
		ITreeContentProvider provider = (ITreeContentProvider)treeViewer.getContentProvider();
		Object parent = provider.getParent(element);
		ConstraintConfiguration config = ((ConstraintConfiguration)parent);

		if(config.getName().equalsIgnoreCase(editedObject.getCurrentObjective().getName())) {
			for(ConstraintInstance instance : constraintInstances) {
				for(String constraintName : editedObject.getCurrentObjective()
						.getActiveConstraints()) {
					if(instance.getConstraintName().equalsIgnoreCase(constraintName) &&
							(hashMap.containsKey(constraintName))) {
						IConstraintInstanceStatus status =
								IConstraintUIService.getInstance().getStatus(instance);
						if(status instanceof SuccessConstraintInstanceStatus) {
							successCount++;
						}
						if(status instanceof FailedConstraintInstanceStatus) {
							failCount++;
						}
					}
				}
			}
			ArrayList<Integer> countList = new ArrayList<Integer>();
			countList.add(new Integer(successCount));
			countList.add(new Integer(failCount));
			countList.add(new Integer(hashMap.size()));
			return countList;
		}
		return null;
	}
}
