/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import static org.eclipse.draw2d.ColorConstants.black;
import static org.eclipse.draw2d.ColorConstants.white;
import static org.eclipse.draw2d.PositionConstants.EAST;
import static org.eclipse.draw2d.PositionConstants.LEFT;
import static org.eclipse.draw2d.PositionConstants.TOP;
import static org.eclipse.gef.EditPolicy.COMPONENT_ROLE;
import static org.eclipse.gef.EditPolicy.GRAPHICAL_NODE_ROLE;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE;
import static org.fortiss.tooling.base.utils.LayoutDataUtils.getNodePosition;

import java.util.ArrayList;
import java.util.List;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.base.model.element.IConnection;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.base.ui.editpart.figure.AntiAliasedEllipse;
import org.fortiss.tooling.base.ui.editpart.figure.TransparentLabel;
import org.fortiss.tooling.base.ui.editpart.policy.DirectConnectionAndReconnectionEditPolicy;
import org.fortiss.tooling.base.ui.editpart.policy.RemoveEditPolicy;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

/**
 * Base class for connector edit parts.
 * 
 * @author hoelzl
 */
public abstract class ConnectorEditPartBase<T extends ILayoutedModelElement & IConnector & INamedCommentedElement>
		extends PositionedEditPartBase<T> {

	/** Constructor. */
	protected ConnectorEditPartBase(T modelElement) {
		super(modelElement);
	}

	/** {@link Font} to be used to render the label returned by {@link #getFigureLabel()}. */
	private static Font FIGURE_LABEL_FONT = createFigureLabelFont();

	/** Helper method to create the font for label returned by {@link #getFigureLabel()}. */
	private static Font createFigureLabelFont() {
		final Font defaultFont = JFaceResources.getDefaultFont();
		// There is at least one FontData
		final String fontName = defaultFont.getFontData()[0].getName();
		return new Font(Display.getCurrent(), fontName, DEFAULT_CONNECTOR_SIZE / 4, SWT.NORMAL);
	}

	/**
	 * Returns the optional one {@link Character} label to be rendered within the circle
	 * representing the connector.
	 */
	protected Character getFigureLabel() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected IFigure createBaseFigure() {
		Figure base = new Figure();
		base.setLayoutManager(new XYLayout());

		Ellipse connector = new AntiAliasedEllipse();
		if(isTarget()) {
			connector.setBackgroundColor(getTargetBackgroundColor());
			connector.setForegroundColor(getTargetForegroundColor());
		} else {
			connector.setBackgroundColor(getSourceBackgroundColor());
			connector.setForegroundColor(getSourceForegroundColor());
		}

		int oneFourth = DEFAULT_CONNECTOR_SIZE / 4;
		int half = DEFAULT_CONNECTOR_SIZE / 2;
		base.add(connector, new Rectangle(oneFourth, oneFourth, half, half));

		if(getFigureLabel() != null) {
			final Label label = new Label(getFigureLabel().toString());
			if(isTarget()) {
				label.setForegroundColor(getTargetForegroundColor());
			} else {
				label.setForegroundColor(getTargetBackgroundColor());
			}
			label.setFont(FIGURE_LABEL_FONT);

			base.add(label, new Rectangle(0, 0, DEFAULT_CONNECTOR_SIZE, DEFAULT_CONNECTOR_SIZE));
		}

		return base;
	}

	/** {@inheritDoc} */
	@Override
	protected Rectangle determineBaseFigureBounds() {
		Point connectorPos = getNodePosition(getModel());
		return new Rectangle(connectorPos.getX(), connectorPos.getY(), DEFAULT_CONNECTOR_SIZE,
				DEFAULT_CONNECTOR_SIZE);
	}

	/** {@inheritDoc} */
	@Override
	protected Label createLabelFigure() {
		Label nameLabel = new TransparentLabel();
		nameLabel.setForegroundColor(getForegroundColor());
		nameLabel.setTextAlignment(LEFT | TOP);
		nameLabel.setTextPlacement(EAST);
		return nameLabel;
	}

	/** {@inheritDoc} */
	@Override
	protected Rectangle determineLabelFigureBounds() {
		// position label right of base figure
		Rectangle textBounds = labelFigure.getTextBounds();
		Rectangle baseBounds = baseFigure.getBounds();
		return new Rectangle(baseBounds.x + baseBounds.width, baseBounds.y, textBounds.width,
				baseBounds.height);
	}

	/** {@inheritDoc} */
	@Override
	protected String determineLabelFigureText() {
		// display name if no connections are present
		if(getModelSourceConnections().isEmpty() && getModelTargetConnections().isEmpty()) {
			return getModel().getName();
		}
		return "";
	}

	/** {@inheritDoc} */
	@Override
	protected boolean useSmallDecorationImage() {
		return true;
	}

	/** Returns foreground color for target role. */
	protected Color getTargetForegroundColor() {
		return getForegroundColor();
	}

	/** Returns background color for target role. */
	protected Color getTargetBackgroundColor() {
		return getBackgroundColor();
	}

	/** Returns foreground color for source role. */
	protected Color getSourceForegroundColor() {
		return getForegroundColor();
	}

	/** Returns background color for source role. */
	protected Color getSourceBackgroundColor() {
		return getForegroundColor();
	}

	/** {@inheritDoc} */
	@Override
	protected Color getForegroundColor() {
		return black;
	}

	/** {@inheritDoc} */
	@Override
	protected Color getBackgroundColor() {
		return white;
	}

	/** {@inheritDoc} */
	@Override
	protected ConnectionAnchor defaultConnectionAnchor() {
		return new ChopboxAnchor(baseFigure);
	}

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();

		refreshSourceConnections();
		refreshTargetConnections();

		for(ConnectionEditPart editPart : getSourceConnections()) {
			editPart.refresh();
		}
		for(ConnectionEditPart editPart : getTargetConnections()) {
			editPart.refresh();
		}
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public List<ConnectionEditPart> getSourceConnections() {
		return super.getSourceConnections();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public List<ConnectionEditPart> getTargetConnections() {
		return super.getTargetConnections();
	}

	/** {@inheritDoc} */
	@Override
	protected List<IConnection> getModelSourceConnections() {
		if(!isTarget()) {
			return filterConnections(getModel().getOutgoing(), getRootHierarchicElement());
		}
		return CollectionUtils.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	protected List<IConnection> getModelTargetConnections() {
		if(isTarget()) {
			return filterConnections(getModel().getIncoming(), getRootHierarchicElement());
		}
		return CollectionUtils.emptyList();
	}

	/** Filters any {@link IConnection}s not owned by the given owner. */
	private List<IConnection> filterConnections(List<IConnection> connections,
			IHierarchicElement owner) {
		List<IConnection> result = new ArrayList<IConnection>();
		for(IConnection connection : connections) {
			if(connection.eContainer() == owner || connection.eContainer() == null) {
				result.add(connection);
			}
		}
		return result;
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(COMPONENT_ROLE, new RemoveEditPolicy());
		installEditPolicy(GRAPHICAL_NODE_ROLE, new DirectConnectionAndReconnectionEditPolicy());
	}

	/** Returns the root element for this connector edit part. */
	protected abstract IHierarchicElement getRootHierarchicElement();

	/**
	 * Returns whether the underlying connector edit part acts as a target
	 * connector in the current view (i.e., edit part).
	 */
	protected abstract boolean isTarget();
}
